/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.logging.Logger;

import javax.activation.DataHandler;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.extension.artifactrepository.api.Artifact;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.ArtifactRepositoryService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.to.AttachmentDescriptor;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = ArtifactRepositoryService.class) })
public class ArtifactRepositoryServiceImpl implements KernelWebService, ArtifactRepositoryService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.ArtifactRepositoryService";

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Controller
    private Component component;

    @Requires(name = "artifactrepository")
    private org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService artifactRepositoryService;

    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    private File uploadPath;

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.start();

        this.uploadPath = new File(this.configurationService.getContainerConfiguration()
                .getWorkDirectoryPath(), "upload");
        if (!this.uploadPath.exists()) {
            this.uploadPath.mkdirs();
        }

        this.log.end();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    public boolean addArtifact(AttachmentDescriptor descriptor) throws PEtALSWebServiceException {
        boolean result = false;
        File out = null;
        DataHandler dh = descriptor.getAttachment();
        if (dh != null) {
            try {
                out = new File(this.uploadPath, "artifactUpload-" + System.currentTimeMillis());
                dh.writeTo(new FileOutputStream(out));

                // try to add this as a repository artifact
                Artifact artifact = new Artifact();
                artifact.setFile(out);

                result = this.artifactRepositoryService.addArtifact(artifact);
                if (result) {
                    out.delete();
                } else {
                    // FIXME : Delete it or move it somewhere in the trash
                    // folder
                    throw new PEtALSWebServiceException(
                            "The file has not been added to the artifact repository service");
                }
            } catch (IOException e) {
                throw new PEtALSWebServiceException(e.getMessage());
            } catch (PetalsException e) {
                throw new PEtALSWebServiceException(e.getMessage());
            }
        }
        return result;
    }

    public String[] getComponents() {
        final List<String> components = this.artifactRepositoryService.getComponentsName();
        return components.toArray(new String[0]);
    }

    public String[] getServiceAssemblies() {
        final List<String> sas = this.artifactRepositoryService.getServiceAssembliesName();
        return sas.toArray(new String[0]);
    }

    public String[] getSharedLibraries() {
        final List<String> sls = this.artifactRepositoryService.getSharedLibrariesName();
        return sls.toArray(new String[0]);
    }

}
