/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.xml.transform.TransformerException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.EndpointService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.to.Endpoint;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;
import com.ebmwebsourcing.easycommons.pooling.PoolException;
import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = EndpointService.class) })
public class EndpointServiceImpl implements KernelWebService, EndpointService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.EndpointService";

    @Controller
    private Component component;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Requires(name = "endpoint")
    private EndpointRegistry endpointRegistry;

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public List<Endpoint> getEndpoints() throws PEtALSWebServiceException {
        List<Endpoint> result = new ArrayList<Endpoint>();

        List<PetalsServiceEndpoint> endpoints = null;
        try {
            endpoints = this.endpointRegistry.getEndpoints();
        } catch (RegistryException e) {
            final String message = "Can not get endpoint list";
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message);
        }

        for (PetalsServiceEndpoint serviceEndpoint : endpoints) {
            Endpoint endpoint = new Endpoint();
            endpoint.setComponent(serviceEndpoint.getLocation().getComponentName());
            endpoint.setContainer(serviceEndpoint.getLocation().getContainerName());
            endpoint.setInterfaces(serviceEndpoint.getInterfaces());
            endpoint.setName(serviceEndpoint.getEndpointName());
            endpoint.setService(serviceEndpoint.getServiceName());
            endpoint.setSubDomain(serviceEndpoint.getLocation().getSubdomainName());

            try {
                final Document description = serviceEndpoint.getDescription();
                if (description != null) {
                    endpoint.setDescription(XMLHelper.createStringFromDOMDocument(description));
                }
            } catch (final TransformerException e) {
                this.log.warning(e.getMessage(), e);
            } catch (final PoolException e) {
                this.log.warning(e.getMessage(), e);
            } catch (final Throwable e) {
                this.log.warning("An error occurs. Skipped", e);
            }

            result.add(endpoint);
        }

        return result;
    }

}
