/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.extension.artifactrepository.api.Artifact;
import org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.InstallationService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.microkernel.api.jbi.management.ComponentInstallationService;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "webservice", signature = InstallationService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class InstallationServiceImpl implements InstallationService, KernelWebService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.InstallationService";

    @Controller
    private Component component;

    @Requires(name = "componentInstallation")
    private ComponentInstallationService componentInstallationService;

    @Requires(name = "artifactrepository")
    private ArtifactRepositoryService artifactRepositoryService;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public boolean install(String componentId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : install with params componentId = " + componentId);
        }

        // get the component from the repository...
        List<Artifact> components = this.artifactRepositoryService.getComponents();
        Iterator<Artifact> iter = components.iterator();
        boolean found = false;
        Artifact artifact = null;
        while (iter.hasNext() && !found) {
            Artifact a = iter.next();
            if ((a.getName() != null) && a.getName().equals(componentId)) {
                artifact = a;
                found = true;
            }
        }

        if (artifact == null) {
            throw new PEtALSWebServiceException("Can not find component " + componentId
                    + " from repository");
        }

        URL url = null;
        try {
            url = artifact.getFile().toURI().toURL();
        } catch (MalformedURLException e) {
            throw new PEtALSWebServiceException(e.getMessage());
        }

        return this.install(url);
    }

    /**
     * {@inheritDoc}
     */
    public boolean install(URL url) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : install with params url = " + url);
        }
        return this.componentInstallationService.install(url);
    }

    /**
     * {@inheritDoc}
     */
    public boolean uninstall(String componentId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : uninstall with params componentId = " + componentId);
        }
        return this.componentInstallationService.uninstall(componentId);
    }
}
