/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import javax.jbi.management.ComponentLifeCycleMBean;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.JBIArtefactsService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.to.ServiceAssembly;
import org.ow2.petals.kernel.ws.api.to.SharedLib;
import org.ow2.petals.microkernel.api.container.ComponentLifeCycle;
import org.ow2.petals.microkernel.api.container.ContainerService;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.SharedLibraryBean;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "webservice", signature = JBIArtefactsService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class JBIArtefactsServiceImpl implements KernelWebService, JBIArtefactsService {
	
    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.JBIArtefactsService";

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Controller
    private Component component;

    @Requires(name = "containerService")
    private ContainerService containerService;

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public List<ServiceAssembly> getServiceAssemblies() throws PEtALSWebServiceException {
        List<ServiceAssembly> result = new ArrayList<ServiceAssembly>();

        Map<String, ServiceAssemblyLifeCycle> sas = this.containerService.getServiceAssemblies();
        if (sas != null) {
            Iterator<String> iter = sas.keySet().iterator();
            while (iter.hasNext()) {
                String saName = iter.next();
                ServiceAssemblyLifeCycle sal = sas.get(saName);

                ServiceAssembly sa = new ServiceAssembly();
                sa.setState(sal.getCurrentState());
                sa.setName(saName);
                if (sal.getServiceAssembly().getIdentification() != null) {
                    sa
                            .setDescription(sal.getServiceAssembly().getIdentification()
                                    .getDescription());
                }
                result.add(sa);
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<org.ow2.petals.kernel.ws.api.to.Component> getComponents()
            throws PEtALSWebServiceException {
        List<org.ow2.petals.kernel.ws.api.to.Component> result = new ArrayList<org.ow2.petals.kernel.ws.api.to.Component>();

        final Map<String, ComponentLifeCycle> lc = this.containerService.getComponentLifecycles();
        for (final Entry<String, ComponentLifeCycle> entry : lc.entrySet()) {
            String name = entry.getKey();
            ComponentLifeCycleMBean clc = entry.getValue();
            org.ow2.petals.kernel.ws.api.to.Component c = new org.ow2.petals.kernel.ws.api.to.Component();
            c.setState(clc.getCurrentState());
            c.setName(name);
            if (this.containerService.isBindingComponent(name)) {
                c.setType(BINDING);
            } else if (this.containerService.isServiceEngine(name)) {
                c.setType(ENGINE);
            }
            result.add(c);
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<SharedLib> getSharedLibraries() throws PEtALSWebServiceException {
        final List<SharedLib> result = new ArrayList<SharedLib>();

        final Map<SharedLibraryBean, SharedLibraryLifeCycle> lc = this.containerService.getSharedLibraries();
        for (final Entry<SharedLibraryBean, SharedLibraryLifeCycle> entry : lc.entrySet()) {
            final SharedLibraryBean slBean = entry.getKey();
            final SharedLibraryLifeCycle slc = entry.getValue();
            final SharedLib sl = new SharedLib();
            sl.setState(slc.getState());
            sl.setName(slBean.getName());
            sl.setVersion(slBean.getVersion());
            result.add(sl);
        }
        return result;
    }
}
