/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.lang.Thread.UncaughtExceptionHandler;
import java.util.concurrent.Callable;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.RuntimeService;
import org.ow2.petals.microkernel.api.admin.PetalsAdminInterface;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = RuntimeService.class) })
public class RuntimeServiceImpl implements KernelWebService, RuntimeService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.RuntimeService";

    @Requires(name = "petalsAdminService")
    private PetalsAdminInterface petalsAdminInterface;

    @Controller
    private Component component;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    @Override
    public Component getComponent() {
        return this.component;
    }

    @Override
    public boolean ping() throws PEtALSWebServiceException {
        return true;
    }

    @Override
    public void shutdownContainer() throws PEtALSWebServiceException {
        // let's do it asynchronously
        executeAsync(new Callable<Object>() {
            @Override
            public Object call() throws Exception {
                synchronized (this) {
                    this.wait(1000);
                }
                RuntimeServiceImpl.this.petalsAdminInterface.shutdownContainer();
                return null;
            }
        }, "Container can not be shutdowned remotely");
    }

    @Override
    public void stopContainer() throws PEtALSWebServiceException {
        executeAsync(new Callable<Object>() {
            @Override
            public Object call() throws Exception {
                synchronized (this) {
                    this.wait(1000);
                }
                RuntimeServiceImpl.this.petalsAdminInterface.stopContainer();
                return null;
            }
        }, "Container can not be stopped remotely");
    }

    private void executeAsync(final Callable<Object> c, final String msg) {
        final Thread t = new Thread() {
            @Override
            public void run() {
                try {
                    c.call();
                } catch (Exception e) {
                    RuntimeServiceImpl.this.log.warning(msg, e);
                }
            }
        };
        t.setUncaughtExceptionHandler(new UncaughtExceptionHandler() {
            @Override
            public void uncaughtException(Thread t, Throwable e) {
                RuntimeServiceImpl.this.log.error(msg, e);
            }
        });
        t.start();
    }
}
