/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.ServiceAssemblyStateService;
import org.ow2.petals.microkernel.api.container.ContainerService;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = ServiceAssemblyStateService.class) })
public class ServiceAssemblyStateServiceImpl implements ServiceAssemblyStateService, KernelWebService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.ServiceAssemblyStateService";

    @Controller
    private Component component;

    @Requires(name = "containerService")
    private ContainerService containerService;

    private LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isShutdown(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isShutdownState();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isStarted(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isStartedState();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isStopped(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isStoppedState();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUnknown(String saId) throws PEtALSWebServiceException {
        ServiceAssemblyLifeCycle sa = this.getServiceAssemblyLifeCycle(saId);
        return sa.isUnknownState();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    private ServiceAssemblyLifeCycle getServiceAssemblyLifeCycle(final String saId)
            throws PEtALSWebServiceException {
        final ServiceAssemblyLifeCycle saAssemblyLifeCycle = this.containerService.getServiceAssemblyByName(saId);
        if (saAssemblyLifeCycle == null) {
            throw new PEtALSWebServiceException("Can not find a service assembly with ID = " + saId);
        }
        return saAssemblyLifeCycle;
    }

}
